/**
 * This work was created by participants in the DataONE project, and is
 * jointly copyrighted by participating institutions in DataONE. For
 * more information on DataONE, see our web site at http://dataone.org.
 *
 *   Copyright ${year}
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dataone.service.mn.tier3.v1;

import java.io.InputStream;

import org.dataone.service.exceptions.IdentifierNotUnique;
import org.dataone.service.exceptions.InsufficientResources;
import org.dataone.service.exceptions.InvalidRequest;
import org.dataone.service.exceptions.InvalidSystemMetadata;
import org.dataone.service.exceptions.InvalidToken;
import org.dataone.service.exceptions.NotAuthorized;
import org.dataone.service.exceptions.NotFound;
import org.dataone.service.exceptions.NotImplemented;
import org.dataone.service.exceptions.ServiceFailure;
import org.dataone.service.exceptions.UnsupportedType;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.Session;
import org.dataone.service.types.v1.SystemMetadata;

/**
 * The DataONE Member Node Tier 3 Storage interface.  This defines an
 * implementation interface for Member Nodes that wish to build an
 * implementation that is compliant with the DataONE service definitions.
 *
 * @author Matthew Jones
 */
public interface MNStorage {

    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.create
     */
    public Identifier create(Identifier pid, InputStream object, 
        SystemMetadata sysmeta) 
    throws IdentifierNotUnique, InsufficientResources, InvalidRequest, InvalidSystemMetadata, 
    	InvalidToken, NotAuthorized, NotImplemented, ServiceFailure, UnsupportedType;


    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.update
     */
    public Identifier update(Identifier pid, InputStream object, 
        Identifier newPid, SystemMetadata sysmeta) 
    throws IdentifierNotUnique, InsufficientResources, InvalidRequest, InvalidSystemMetadata, 
        InvalidToken, NotAuthorized, NotImplemented, ServiceFailure, UnsupportedType,
        NotFound;

    
    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.delete
     */
    public Identifier delete(Identifier pid)
    throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented;

    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.archive
     */
    public Identifier archive(Identifier pid)
    throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented;    
    
    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MNStorage.generateIdentifier
     */
    public Identifier generateIdentifier(String scheme, String fragment)
    	throws InvalidToken, ServiceFailure,
            NotAuthorized, NotImplemented, InvalidRequest;
    
    /////
    
    
    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.create
     */
    @Deprecated
    public Identifier create(Session session, Identifier pid, InputStream object, 
        SystemMetadata sysmeta) 
    throws IdentifierNotUnique, InsufficientResources, InvalidRequest, InvalidSystemMetadata, 
    	InvalidToken, NotAuthorized, NotImplemented, ServiceFailure, UnsupportedType;


    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.update
     */
    @Deprecated
    public Identifier update(Session session, Identifier pid, InputStream object, 
        Identifier newPid, SystemMetadata sysmeta) 
    throws IdentifierNotUnique, InsufficientResources, InvalidRequest, InvalidSystemMetadata, 
        InvalidToken, NotAuthorized, NotImplemented, ServiceFailure, UnsupportedType,
        NotFound;

    
    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.delete
     */
    @Deprecated
    public Identifier delete(Session session, Identifier pid)
    throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented;

    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MN_storage.archive
     */
    @Deprecated
    public Identifier archive(Session session, Identifier pid)
    throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented;    
    
    /**
     * @see https://purl.dataone.org/architecturev2/apis/MN_APIs.html#MNStorage.generateIdentifier
     */
    @Deprecated
    public Identifier generateIdentifier(Session session, String scheme, String fragment)
    	throws InvalidToken, ServiceFailure,
            NotAuthorized, NotImplemented, InvalidRequest;
}
